"use strict";
const {
    classes: Cc,
    interfaces: Ci,
    results: Cr,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cc["@mozilla.org/moz/jssubscript-loader;1"].createInstance(Ci.mozIJSSubScriptLoader).loadSubScript(__SCRIPT_URI_SPEC__.replace(/[^\/]+$/, "appName.js"));
let application;
let extensionEnabled = false;
let extensionUpdated = false;
const core = {
    get appName() {
        return APP_NAME;
    },
    get extensionId() {
        return this._bootstrapData.id;
    },
    get extensionVersion() {
        return this._bootstrapData.version;
    },
    init: function (bootstrapData) {
        this._defineGetters();
        this._saveBootstrapData(bootstrapData);
        this._runApplication();
    },
    finalize: function (isUpdate) {
        this._stopApplication(isUpdate);
    },
    _defineGetters: function () {
        XPCOMUtils.defineLazyGetter(this, "appProfileDir", () => {
            let profileDir = getAppProfileDir();
            if (!profileDir.exists()) {
                try {
                    profileDir.create(Ci.nsIFile.DIRECTORY_TYPE, parseInt("0755", 8));
                } catch (e) {
                }
            }
            return profileDir;
        });
        XPCOMUtils.defineLazyGetter(this, "buildConfigFile", () => {
            let buildDataFile = this.appProfileDir;
            buildDataFile.append("config.json");
            if (!buildDataFile.exists()) {
                this._copyFileByPath(this.appContentPath + buildDataFile.leafName, buildDataFile);
            }
            return buildDataFile;
        });
        XPCOMUtils.defineLazyGetter(this, "appContentPath", () => "chrome://" + APP_NAME + "/content/");
        XPCOMUtils.defineLazyGetter(this, "appLocalePath", () => "chrome://" + APP_NAME + "/locale/");
    },
    _saveBootstrapData: function (data) {
        this._bootstrapData = data;
    },
    _copyFileByPath: function (sourceFilePath, targetFile) {
        let stream = this.getFileInputStream(sourceFilePath);
        if (stream) {
            this._writeStreamToFile(stream, targetFile);
        }
    },
    getFileInputStream: function (filePath) {
        let channel = this._getChannel(filePath);
        try {
            return channel.open();
        } catch (e) {
        }
        return null;
    },
    _getChannel: function (url) {
        try {
            let uri = Services.io.newURI(url, null, null);
            return "newChannelFromURI" in Services.io ? Services.io.newChannelFromURI(uri) : Services.io.newChannelFromURIWithLoadInfo(uri, null);
        } catch (e) {
        }
        return null;
    },
    _writeStreamToFile: function (stream, file) {
        let fos = this._createFileOutputStream(file);
        if (fos) {
            let bos = Cc["@mozilla.org/network/buffered-output-stream;1"].createInstance(Ci.nsIBufferedOutputStream);
            try {
                let available = stream.available();
                bos.init(fos, available);
                bos.writeFrom(stream, available);
            } finally {
                try {
                    bos.close();
                } catch (e) {
                }
            }
        }
        try {
            stream.close();
        } catch (e) {
        }
    },
    _createFileOutputStream: function (file) {
        let fileOutputStream = null;
        if (this._ensureFileExists(file)) {
            try {
                fileOutputStream = Cc["@mozilla.org/network/file-output-stream;1"].createInstance(Ci.nsIFileOutputStream);
                fileOutputStream.init(file, 2 | 8 | 32, parseInt("0644", 8), 0);
            } catch (e) {
            }
        }
        return fileOutputStream;
    },
    _ensureFileExists: function (file) {
        if (!file.exists()) {
            try {
                file.create(Ci.nsIFile.NORMAL_FILE_TYPE, parseInt("0644", 8));
            } catch (e) {
                return false;
            }
        }
        return true;
    },
    _runApplication: function () {
        let scope = Object.create(null);
        Cu.unload(this.appContentPath + "application.jsm");
        Cu.import(this.appContentPath + "application.jsm", scope);
        application = scope.application;
        application.start(this, {
            newStart: extensionEnabled,
            updateStart: extensionUpdated
        });
    },
    _stopApplication: function (isUpdate) {
        application.rollback(isUpdate);
        application = null;
    }
};
function startup(data, reason) {
    extensionEnabled = reason === ADDON_ENABLE || reason === ADDON_INSTALL;
    core.init(data);
}
function shutdown(data, reason) {
    if (reason === ADDON_DISABLE || reason === ADDON_UNINSTALL) {
        core.finalize();
    } else if (reason === ADDON_UPGRADE || reason === ADDON_DOWNGRADE) {
        core.finalize(true);
    }
}
function install(data, reason) {
    extensionUpdated = reason === ADDON_UPGRADE;
}
function uninstall(data, reason) {
    if (reason === ADDON_UNINSTALL) {
        try {
            getAppProfileDir().remove(true);
        } catch (e) {
        }
        let prefBranchPath = "extensions." + data.id + ".";
        Services.prefs.getBranch(prefBranchPath).getChildList("", {}).forEach(function (key) {
            Services.prefs.clearUserPref(prefBranchPath + key);
        });
    }
}
function getAppProfileDir() {
    let profileDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
    profileDir.append(APP_NAME);
    return profileDir;
}