"use strict";
EXPORTED_SYMBOLS.push("patterns");
const patterns = {};
patterns.NotificationSource = function () {
    this._listeners = {};
};
patterns.NotificationSource.prototype = {
    constructor: patterns.NotificationSource,
    addListener: function (topic, listener) {
        let topicListeners = this._listeners[topic] || (this._listeners[topic] = []);
        if (topicListeners.indexOf(listener) >= 0) {
            return;
        }
        topicListeners.push(listener);
        this._listenerAdded(topic, listener);
    },
    removeListener: function (topic, listener) {
        let topicListeners = this._listeners[topic] || (this._listeners[topic] = []);
        if (!topicListeners) {
            return;
        }
        let listenerIdx = topicListeners.indexOf(listener);
        if (listenerIdx < 0) {
            return;
        }
        topicListeners.splice(listenerIdx, 1);
        if (topicListeners.length < 1) {
            delete this._listeners[topic];
        }
        this._listenerRemoved(topic, listener);
    },
    removeAllListeners: function () {
        if (this._hasListeners) {
            this._listeners = {};
            this._listenerRemoved();
        }
    },
    _listeners: null,
    get _hasListeners() {
        for (let [topic, list] in Iterator(this._listeners)) {
            if (list && list.length) {
                return true;
            }
        }
        return false;
    },
    _getListeners: function (topic) {
        return this._listeners[topic] || [];
    },
    _notifyListeners: function (topic, data) {
        let topicListeners = this._getListeners(topic);
        topicListeners.forEach(function (listener) {
            try {
                if (this._listeners[topic].indexOf(listener) != -1) {
                    listener.observe(this, topic, data);
                }
            } catch (e) {
                Cu.reportError("Could not notify event listener. " + e + "\n" + e.stack);
            }
        }, this);
    },
    _listenerAdded: function (topic, listener) {
    },
    _listenerRemoved: function (topic, listener) {
    }
};
patterns.NotificationSource.objectMixIn = function (object) {
    let notificationSourceInstance = new this();
    copy(Object.getPrototypeOf(notificationSourceInstance), object);
    copy(notificationSourceInstance, object);
    function copy(source, target) {
        Object.keys(source).forEach(function (aProp) {
            if (aProp === "constructor") {
                return;
            }
            let descriptor = Object.getOwnPropertyDescriptor(source, aProp);
            Object.defineProperty(target, aProp, descriptor);
        });
    }
};