"use strict";
EXPORTED_SYMBOLS.push("sysutils");
const sysutils = {};
sysutils.Timer = function (aCallback, aDelay, aRepeating, aMaxTimes) {
    this.callback = aCallback;
    this.repeatingDelay = null;
    if (typeof aRepeating == "number") {
        if (aRepeating > 0) {
            this.repeatingDelay = aRepeating;
        }
    } else if (typeof aRepeating == "boolean") {
        if (aDelay > 0) {
            this.repeatingDelay = aDelay;
        }
    }
    this.timesCounter = 0;
    this.maxTimes = typeof aMaxTimes == "number" && aMaxTimes > 0 ? aMaxTimes : null;
    this.timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    let type = this.repeatingDelay ? this.timer.TYPE_REPEATING_SLACK : this.timer.TYPE_ONE_SHOT;
    this.timer.initWithCallback(this, aDelay, type);
    this._initTimestamp = Date.now();
    Services.obs.addObserver(this, "wake_notification", false);
};
sysutils.Timer.prototype = {
    get isRunning() {
        return Boolean(this.timer);
    },
    cancel: function () {
        if (!this.timer) {
            return;
        }
        Services.obs.removeObserver(this, "wake_notification");
        this.timer.cancel();
        this.timer = null;
        this.callback = null;
        this._initTimestamp = null;
    },
    notify: function () {
        let result = this.callback();
        this.timesCounter++;
        this._initTimestamp = Date.now();
        if (this.repeatingDelay) {
            this.delay = this.repeatingDelay;
        }
        if (!this.repeatingDelay || this.maxTimes && this.timesCounter >= this.maxTimes) {
            this.cancel();
        }
        return result;
    },
    get delay() {
        return this.timer && this.timer.delay || undefined;
    },
    set delay(val) {
        if (this.timer) {
            this.timer.delay = val;
        }
    },
    observe: function (aSubject, aTopic, aData) {
        if (aTopic !== "wake_notification") {
            return;
        }
        if (!this.isRunning) {
            return;
        }
        let runTime = Date.now() - this._initTimestamp;
        if (runTime <= 0) {
            return;
        }
        this.delay = Math.max(0, this.timer.delay - runTime);
    },
    _initTimestamp: null,
    QueryInterface: XPCOMUtils.generateQI([
        Ci.nsISupports,
        Ci.nsITimerCallback,
        Ci.nsIObserver
    ])
};