"use strict";
const EXPORTED_SYMBOLS = ["bannerId"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
let Lib;
const bannerId = {
    get app() {
        return this._app;
    },
    get id() {
        return this._getBannerId();
    },
    init: function (app) {
        this._app = app;
        Lib = app.Lib;
    },
    _getBannerId: function () {
        if (this._bannerId === undefined) {
            if (this.app.isNewStart) {
                this._retrieveBannerId();
            }
            let bannerId = null;
            let bnrdFile = this._getBannerIdFile();
            if (bnrdFile.exists()) {
                try {
                    bannerId = this._validateBannerId(Lib.fileutils.readTextFile(bnrdFile));
                } catch (e) {
                }
            }
            this._bannerId = bannerId;
        }
        return this._bannerId;
    },
    _getBannerIdFile: function () {
        let bnrdFile = this.app.extensionDataDir;
        bnrdFile.append("bnrd");
        return bnrdFile;
    },
    _validateBannerId: function (bannerId) {
        bannerId = (bannerId || "").trim();
        if (bannerId && bannerId.length <= 100) {
            return bannerId;
        }
        return null;
    },
    _retrieveBannerId: function () {
        let bnrdFile = this._getBannerIdFile();
        if (!bnrdFile.exists()) {
            const FIELD_NAME = "bnrd";
            let yCookieModule = this.app._modules.yCookie;
            let removeCookieField = false;
            this._getLookUpDomains().some(domain => {
                let fieldValue = yCookieModule.getSessionCookieFieldValue(FIELD_NAME, domain);
                if (fieldValue) {
                    removeCookieField = true;
                    if (this._validateBannerId(fieldValue)) {
                        try {
                            Lib.fileutils.writeTextFile(bnrdFile, fieldValue);
                        } catch (e) {
                        }
                        return true;
                    }
                }
            });
            if (removeCookieField) {
                yCookieModule.setSessionCookieFieldValue(FIELD_NAME, null);
            }
        }
    },
    _getLookUpDomains: function () {
        let domains = [];
        switch (this.app.brandID) {
        case "yandex":
            domains = [
                ".yandex.ru",
                ".yandex.kz",
                ".yandex.by"
            ];
            break;
        case "ua":
            domains = [".yandex.ua"];
            break;
        case "tb":
            domains = [".yandex.com.tr"];
            break;
        }
        return domains;
    }
};