"use strict";
const EXPORTED_SYMBOLS = ["brander"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu,
    results: Cr
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const brander = {
    init: function (application) {
        this._application = application;
    },
    finalize: function () {
        this._application = null;
    },
    get SUGGESTED_BRAND_ID() {
        let brandId = "yandex";
        switch (this.PREFERED_LOCALE) {
        case "uk":
            brandId = "ua";
            break;
        case "tr":
            brandId = "tb";
            break;
        }
        return brandId;
    },
    get SUGGESTED_PRESET_URL() {
        let presetURL = this._application.preferences.get("default.preset.url", null);
        if (!presetURL) {
            switch (this.SUGGESTED_BRAND_ID) {
            case "ua":
                presetURL = "http://bar.yandex.ru/presets/ua.xml";
                break;
            case "tb":
                presetURL = "https://download.yandex.ru/bar/wwt/presets/tb.xml";
                break;
            default:
                presetURL = "http://bar.yandex.ru/presets/default.xml";
                break;
            }
            this._application.preferences.set("default.preset.url", presetURL);
        }
        return presetURL;
    },
    get PREFERED_LOCALE() {
        let locale = this._application.preferences.get("default.preferedLocale", null);
        locale = this._validateLocaleValue(locale);
        if (!locale) {
            locale = this._determineLocale();
        }
        this._application.preferences.set("default.preferedLocale", locale);
        return locale;
    },
    _determineLocale: function () {
        let locale;
        if (!locale) {
            let country = this._application.Lib.WinReg.read("HKCU", "Control Panel\\International", "iCountry");
            let countries = {
                "7": "ru",
                "49": "de",
                "62": "in",
                "90": "tr",
                "375": "be",
                "380": "uk",
                "705": "kk"
            };
            locale = countries[country];
            locale = this._validateLocaleValue(locale, "region");
        }
        if (!locale) {
            locale = Services.locale.getLocaleComponentForUserAgent().substr(0, 2);
            locale = this._validateLocaleValue(locale, "lang");
        }
        if (!locale) {
            let xulChromeReg = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
            locale = xulChromeReg.getSelectedLocale("global").substr(0, 2);
            locale = this._validateLocaleValue(locale, "region");
        }
        if (!locale) {
            let countryCode = this._application.Lib.Preferences.get("browser.search.countryCode", "");
            countryCode = String(countryCode).toLowerCase();
            let countryCodes = {
                "by": "be",
                "de": "de",
                "id": "in",
                "kz": "kk",
                "ru": "ru",
                "tr": "tr",
                "ua": "uk"
            };
            locale = countryCodes[countryCode];
            locale = this._validateLocaleValue(locale, "region");
        }
        if (!locale) {
            locale = "ru";
        }
        return locale;
    },
    _validateLocaleValue: function (locale, type) {
        locale = String(locale || "").toLowerCase();
        let validLocales = [
            "de",
            "in",
            "ru",
            "tr",
            "uk"
        ];
        if (type === "region") {
            validLocales = validLocales.concat([
                "be",
                "kk"
            ]);
        }
        return validLocales.indexOf(locale) === -1 ? null : locale;
    }
};