"use strict";
const EXPORTED_SYMBOLS = ["homepageManipulator"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const BROWSER_STARTUP_PAGE_PREF = "browser.startup.page";
const BROWSER_STARTUP_HOMEPAGE_PREF = "browser.startup.homepage";
const PREFS = {
    pageType: "type",
    pageURL: "url"
};
const HOSTS_FOR_BRANDING = {
    yandex: "yandex.ru",
    tb: "yandex.com.tr",
    ua: "yandex.ua"
};
let Lib;
const homepageManipulator = {
    get DEFAULT_HOMEPAGE_TYPE() {
        return 1;
    },
    get CLID_TYPE() {
        return 5;
    },
    get app() {
        return this._app;
    },
    init: function (app) {
        this._active = true;
        this._app = app;
        Lib = app.Lib;
        if (this.app.isNewStart) {
            this._setBrowserHomepage();
        } else if (this.app.isUpdate) {
            this.app._modules.installInfo.wasInternalUpdate(internal => {
                if (!internal) {
                    this._setBrowserHomepage();
                }
            });
        }
    },
    rollback: function () {
        this._restoreOriginalUserHomepageSettings();
    },
    finalize: function () {
        this._active = false;
    },
    _setBrowserHomepage: function () {
        if (this._active) {
            this._browserHomepage = new BrowserHomepage();
            this._changeBrowserHomepageType();
            this._changeBrowserHomepage();
        }
    },
    _active: false,
    _browserHomepage: null,
    get _prefsPrefix() {
        delete this._prefsPrefix;
        this._prefsPrefix = "extensions." + this.app.id + ".homepage.";
        return this._prefsPrefix;
    },
    _changeBrowserHomepageType: function () {
        if (this._shouldOverrideBrowserHomepageType()) {
            this._overrideBrowserHomepageType();
        }
    },
    _changeBrowserHomepage: function () {
        if (this._shouldOverrideBrowserHomepage()) {
            this._overrideBrowserHomepage();
        }
    },
    _shouldOverrideBrowserHomepageType: function () {
        return this._browserHomepage.startsWithBlankPage();
    },
    _overrideBrowserHomepageType: function () {
        this._rememberUserHomepageType();
        this._setBrowserHomepageTypeSetting(this.DEFAULT_HOMEPAGE_TYPE);
    },
    _rememberUserHomepageType: function () {
        let prefName = this._prefsPrefix + PREFS.pageType;
        let rememberedSetting = this._getOriginalUserSettings(prefName);
        if (!rememberedSetting.fromType) {
            rememberedSetting.fromType = this._browserHomepage.type;
            this._saveObjectToPref(this._prefsPrefix + PREFS.pageType, rememberedSetting);
        }
    },
    _setBrowserHomepageTypeSetting: function (type) {
        this._setPref(BROWSER_STARTUP_PAGE_PREF, type);
    },
    _shouldOverrideBrowserHomepage: function () {
        return !this._browserHomepage.getUrlArray().some(url => {
            return this._isYandexAffiliatedURI(this._createURI(url));
        });
    },
    _isYandexAffiliatedURI: function (URI) {
        let isYandex = false;
        try {
            let baseDomain = Services.eTLD.getBaseDomain(URI);
            if (baseDomain.indexOf("yandex.") === 0) {
                let host = URI.host;
                isYandex = host === baseDomain || host === "www." + baseDomain;
            } else {
                isYandex = baseDomain === "ya.ru";
            }
        } catch (e) {
        }
        return isYandex;
    },
    _createURI: function (url) {
        return Services.uriFixup.createFixupURI(url, Services.uriFixup.FIXUP_FLAG_USE_UTF8);
    },
    _overrideBrowserHomepage: function () {
        let toURL = this._generateHomepageURL();
        this._rememberUserHomepage(toURL);
        this._setBrowserHomepageSetting(toURL);
    },
    _generateHomepageURL: function () {
        let host = this._getBrandedHost();
        let clid = this._getBrandedClid();
        return "http://www." + host + "/" + (clid ? "?clid=" + clid : "");
    },
    _getBrandedHost: function () {
        return HOSTS_FOR_BRANDING[this.app.brandID] || "yandex.ru";
    },
    _getBrandedClid: function () {
        return this.app.getClid(this.CLID_TYPE);
    },
    _rememberUserHomepage: function (toURL) {
        let prefName = this._prefsPrefix + PREFS.pageURL;
        let rememberedUserHomepageSettings = this._getOriginalUserSettings(prefName);
        if (!rememberedUserHomepageSettings.fromURL) {
            rememberedUserHomepageSettings.fromURL = this._browserHomepage.url;
        }
        rememberedUserHomepageSettings.toURL = toURL;
        this._saveObjectToPref(prefName, rememberedUserHomepageSettings);
    },
    _setBrowserHomepageSetting: function (url) {
        this._setPref(BROWSER_STARTUP_HOMEPAGE_PREF, url);
    },
    _getOriginalUserSettings: function (settingName) {
        let prefValue = this._getParsedObjectFromPref(settingName);
        return prefValue && typeof prefValue === "object" && !Array.isArray(prefValue) ? prefValue : Object.create(null);
    },
    _getParsedObjectFromPref: function (prefName) {
        try {
            return JSON.parse(Lib.Preferences.get(prefName, undefined));
        } catch (e) {
            return null;
        }
    },
    _saveObjectToPref: function (prefName, object) {
        try {
            this._setPref(prefName, JSON.stringify(object));
        } catch (e) {
        }
    },
    _setPref: function (prefName, prefValue) {
        if (typeof prefValue === "boolean" || prefValue) {
            Lib.Preferences.set(prefName, prefValue);
        } else {
            Lib.Preferences.reset(prefName);
        }
    },
    _restoreOriginalUserHomepageSettings: function () {
        let pageURLPrefName = this._prefsPrefix + PREFS.pageURL;
        let pageTypePrefName = this._prefsPrefix + PREFS.pageType;
        let rememberedSetting = this._getOriginalUserSettings(pageURLPrefName);
        let currentHomepage = BrowserHomepage.getBrowserHomepage();
        if (rememberedSetting.toURL === currentHomepage) {
            this._setBrowserHomepageSetting(rememberedSetting.fromURL);
            if (BrowserHomepage.getBrowserHomepageType() === this.DEFAULT_HOMEPAGE_TYPE) {
                let rememberedHomepageType = this._getOriginalUserSettings(pageTypePrefName);
                if (rememberedHomepageType.fromType) {
                    this._setPref(BROWSER_STARTUP_PAGE_PREF, rememberedHomepageType.fromType);
                }
            }
        }
        this._setPref(pageURLPrefName, null);
        this._setPref(pageTypePrefName, null);
    }
};
function BrowserHomepage() {
    this.type = BrowserHomepage.getBrowserHomepageType();
    this.url = BrowserHomepage.getBrowserHomepage();
}
BrowserHomepage.getBrowserHomepage = function () {
    let currentHP = getLocalizedPref(BROWSER_STARTUP_HOMEPAGE_PREF, null);
    if (!currentHP) {
        let configBundle = Services.strings.createBundle("chrome://branding/locale/browserconfig.properties");
        currentHP = configBundle.GetStringFromName(BROWSER_STARTUP_HOMEPAGE_PREF);
    }
    return currentHP;
};
BrowserHomepage.getBrowserHomepageType = function (defaultValue) {
    return Lib.Preferences.get(BROWSER_STARTUP_PAGE_PREF, defaultValue);
};
BrowserHomepage.startsWithBlankPage = function (startPageValue) {
    return startPageValue === 0;
};
BrowserHomepage.prototype.startsWithBlankPage = function () {
    return BrowserHomepage.startsWithBlankPage(this.type);
};
BrowserHomepage.prototype.getUrlArray = function () {
    return this.url.split("|");
};
function getLocalizedPref(aPrefName, aDefault) {
    try {
        return Services.prefs.getComplexValue(aPrefName, Ci.nsIPrefLocalizedString).data;
    } catch (ex) {
    }
    return aDefault;
}